// $Id: quickfile.js,v 1.10 2004/11/03 17:27:21 Paul Tomlin Exp $

quickfile = 
{
	consoleService: null,
	
	// source and destination folders
	srcFolder: null,
	dstFolder: null,

	// selected message URIs
	messages: null,

	// user preferences
	prefs: null,
	
	init: function ()
	{
		// get the user preferences
		this.prefs = qfGetPrefs();
		
		if (this.prefs.debug.enable)
			qfDump('quickfile::init');
		
		// get the selected message URIs
		this.messages = window.opener.GetSelectedMessages();
		if (! (this.messages && this.messages.length))
		{
			if (this.prefs.debug.enable)
				qfDump('quickfile::init - no messages selected');
			
			// apparently there are no selected messages
			window.alert('Please select a message or two first!');
			
			// close the window
			window.close();
			
			return false;
		}
		
		// get the folder of the first selected message
		this.srcFolder = qfGetFolderFromMessageURI(this.messages[0]);
		if (! this.srcFolder)
		{
			if (this.prefs.debug.enable)
				qfDump('quickfile::init - failed getting nsIMsgFolder from message URI "' + this.messages[0] + '"');
			
			// failed to get the source folder from the selected messages
			window.alert('QuickFile cannot determine which folder contains your selected message(s)');
			
			// close the window
			window.close();
			
			return false;
		}
		
		// if there is a default folder then use it
		if (this.prefs.defaultfolder.enable && this.prefs.defaultfolder.folder != null)
		{
			// fill in the text field
			document.getElementById('find-text').value = qfGetNameFromFolderURI(this.prefs.defaultfolder.folder);
			
			// set the destination folder
			this.dstFolder = this.prefs.defaultfolder.folder;
		}
		
		// create a new AutoCompleteSession
		var acs = new QuickFileACS();
		
		acs.searchRootURI = this.prefs.search.root;
		acs.caseSensitive = this.prefs.search.casesensitive;
		acs.expandSpaces  = this.prefs.search.expandspaces;
		
		// the callback to use when an entry is selected from the popup
		acs.onAutoCompleteCallbackObj = this;
		acs.onAutoCompleteCallbackFunc = this.onCompleteLookup;
		
		// debugging?
		acs.debug = this.prefs.debug.enable;
		
		if (! acs.prepare())
		{
			// preparation failed, which would normally indicate
			// that the user options are not compatible with this
			// set of circumstances
			if (this.prefs.debug.enable)
				qfDump('quickfile::init - preparation failed');
			
			// silently close the window
			window.close();
			
			return false;
		}
		
		document.getElementById('find-text').addSession(acs);
		
		// hide the progress box
		//document.getElementById('progress-box').hidden = true;
		
		// disable the OK button unless there is a known destination
		if (! this.dstFolder)
			document.documentElement.getButton('accept').disabled = true;
		
		return true;
	},
	
	onCompleteLookup: function (name, uri)
	{
		if (this.prefs.debug.enable)
			qfDump('quickfile::onCompleteLookup("' + name + '", "' + uri + '")');
		
		// it is possible to get NULL values, so ignore them
		if (! name)
		{
			if (this.prefs.debug.enable)
				qfDump('quickfile::onCompleteLookup - incomplete lookup [name not supplied], ignoring it');
				
			return;
		}

		if (! uri)
		{
			if (this.prefs.debug.enable)
				qfDump('quickfile::onCompleteLookup - incomplete lookup [uri not supplied], ignoring it');
				
			return;
		}
		
		// RDF service
		var rdfService = Components.classes['@mozilla.org/rdf/rdf-service;1']
							.getService(Components.interfaces.nsIRDFService);
		
		try
		{
			this.dstFolder = rdfService.GetResource(uri)
								.QueryInterface(Components.interfaces.nsIMsgFolder);
		}
		catch (e)
		{
			if (this.prefs.debug.enable)
				qfDump('quickfile::onCompleteLookup - failed getting nsIMsgFolder from ' + uri + "\n" + e);
			
			// there was an error getting an nsIMsgFolder from the URI
			window.alert('An internal error occured retrieving an nsIMsgFolder for "' + uri + '"');
			
			// reset
			this.dstFolder = null;
			
			return;
		}
		
		// are the source and destination the same
		if (this.srcFolder.URI == this.dstFolder.URI)
		{
			if (this.prefs.debug.enable)
				qfDump('quickfile::onCompleteLookup - destination folder is the same as source folder');
			
			// just bail, we can't use that
			return;
		}
		
		// the lookup was complete, so we should have a real name and uri
		//  to sork with if the user selects it....
		document.documentElement.getButton('accept').disabled = false;
	},
	
	// called when the user enters anything into the text field
	onTextInput: function ()
	{
		if (this.prefs.debug.enable)
			qfDump('quickfile::onTextInput');
		
		// disable the OK button. If the text entered is actually
		//  valid, a later event (onCompleteLookup) will reset this state
		document.documentElement.getButton('accept').disabled = true;
		
		return true;
	},
	
	onTextCommand: function ()
	{
		if (this.prefs.debug.enable)
			qfDump('quickfile::onTextCommand');
		
		// accept the dialog, so long as it's valid...
		if (! document.documentElement.getButton("accept").disabled)
			document.documentElement.acceptDialog();
		else
		{
			if (this.prefs.debug.enable)
			{
				qfDump('quickfile::onTextCommand - dialog accept disabled');
			
				if (this.srcFolder)
					qfDump('                         srcFolder.URI = "' + this.srcFolder.URI + '"');
				else
					qfDump('                         srcFolder undefined');

				if (this.dstFolder)
					qfDump('                         dstFolder.URI = "' + this.dstFolder.URI + '"');
				else
					qfDump('                         dstFolder undefined');
			}
		}
			
		return true;
	},
	
	// called when the user presses ESC to revert the text back to what
	//  they actually typed.
	onTextRevert: function ()
	{
		if (this.prefs.debug.enable)
			qfDump('quickfile::onTextRevert');
		
		document.documentElement.getButton('accept').disabled = true;
		
		return true;
	},
	
	onOk: function ()
	{
		if (this.prefs.debug.enable)
			qfDump('quickfile::onOk');
	
		// rudimentary check that we have all the details we need
		if (! (this.srcFolder && this.dstFolder && this.messages.length))
		{
			if (! this.srcFolder && this.prefs.debug.enable)
				qfDump('quickfile::onOk - srcFolder undefined');
			
			if (! this.dstFolder && this.prefs.debug.enable)
				qfDump('quickfile::onOk - dstFolder undefined');
				
			if (! this.messages.length && this.prefs.debug.enable)
				qfDump('quickfile::onOk - messages undefined');
				
			alert('An error occured');
			
			// close the window
			return true;
		}
		
		if (this.srcFolder.URI == this.dstFolder.URI)
		{
			if (this.prefs.debug.enable)
				qfDump('quickfile::onOk - message destination same as source');
			
			window.alert("You can't move messages to the same folder...");
			
			// leave the window open
			return false;
		}

		// set a wait cursor on the window
		window.setCursor('wait');
		
		// disable input
		document.documentElement.getButton('accept').disabled = true;
		document.documentElement.getButton('cancel').disabled = true;
	
		// hide the textfield and show the progress bar
		document.getElementById('search-box').hidden = true;
		document.getElementById('progress-box').hidden = false;
		document.getElementById('progress-box').value = 1;

		// get the URI for the next message if this option is enabled
		//  do this now otherwise 'next' will have no context since we
		//  are moving the selected messages away
		if (this.prefs.autoselectnext)
		{
			// select the next message
			window.opener.goDoCommand("cmd_nextMsg");
			
			// get the URI for that message
			this.nextMessageURI = window.opener.GetSelectedMessages()[0];
			
			// when we are going to move the last message from a folder, the next message
			//  is actually the same as the selected one(s), so check for this
			//  or we will later end up trying to select a message we have just moved
			// .. or, put another way..
			// check that we are not going to try to later select
			//  one of the messages we are going to move
			for (var i = 0; i < this.messages.length; i++)
			{
				if (this.nextMessageURI == this.messages[i])
				{
					this.nextMessageURI = undefined;
					break;
				}
			}
			
			// reselect user-selected messages (just in case this move fails)
			// TODO: this is currently broken as it only selects the last one..
			for (var i = 0; i < this.messages.length; i++)
			{
				window.opener.SelectMessage(this.messages[i]);
			}
		}

		// do the move/copy operation
		if (! this.doMove())
		{
			if (this.prefs.debug.enable)
				qfDump('quickfile::onOk - doMove failed');
			
			window.alert('There was an error processing your request');

			// close the window
			return true;
		}

		// keep the window open, will be closed when
		//  the copy is completed
		return false;
	},
	
	onCancel: function ()
	{
		if (this.prefs.debug.enable)
			qfDump('quickfile::onCancel');
		
		return true;
	},
	
	doMove: function ()
	{
		if (this.prefs.debug.enable)
			qfDump('quickfile::doMove');
		
		// messenger instance
		var messenger = Components.classes['@mozilla.org/messenger;1']
							.createInstance(Components.interfaces.nsIMessenger);
							
		// nsISupportsArray needed for the CopyMessages op
		var messagesToMove = Components.classes['@mozilla.org/supports-array;1']
								.createInstance(Components.interfaces.nsISupportsArray);
		
		var isMove = ! this.prefs.debug.copynotmove;
		
		for (var i = 0; i < this.messages.length; i++)
		{
			var message = this.messages[i];
			
			// each message uri starts with something like imap-message:// or similar
			//  each type needs to use a different service
			var scheme = message.substr(0, message.indexOf('://'));
			
			if (scheme == 'news-message')
			{
				// nntp messages always get copied
				isMove = false;
			}
			
			// the message service to use for this message
			var service = messenger.messageServiceFromURI(message);
			if (! service)
			{
				if (this.prefs.debug.enable)
					qfDump('quickfile::doMove - no service for message scheme');
					
				return false;
			}
			
			// get a nsIMsgHdr from the uri
			message = service.messageURIToMsgHdr(message);
			if (! message)
			{
				if (this.prefs.debug.enable)
					qfDump('quickfile::doMove - no message header from service');
					
				return false;
			}
			
			// add it to the list to move later
			messagesToMove.AppendElement(message);
		}
		
		var msgCopyService = Components.classes['@mozilla.org/messenger/messagecopyservice;1']
								.getService(Components.interfaces.nsIMsgCopyService);
		if (this.prefs.debug.enable)
		{
			qfDump('quickfile::doMove - from: ' + this.srcFolder.URI + "\n" +
				   '                      to: ' + this.dstFolder.URI + "\n" +
				   '                  isMove: ' + isMove);
		}
		
		// debugging?
		this.msgCopyListener.debug = this.prefs.debug.enable;
		
		msgCopyService.CopyMessages(this.srcFolder, messagesToMove, this.dstFolder, isMove, this.msgCopyListener, null, false);
		
		return true;
	},
	
	onMoveComplete: function (status)
	{
		// see if the status is OK
		if (status != 0)
		{
			if (this.prefs.debug.enable)
				qfDump('quickfile::onMoveComplete - status non-zero');
			
			// notify the user
			window.alert(
				'There was an error ' + 
				(this.prefs.debug.copynotmove ? 'copying' : 'moving') + 
				' your messages'
			);
		}
		
		// select the next message
		if (this.prefs.autoselectnext && this.nextMessageURI)
			window.opener.SelectMessage(this.nextMessageURI);
		
		window.close();
	},
	
	msgCopyListener:
	{
		debug: false,
		
		OnProgress: function (progress, progressMax)
		{
			if (this.debug)
				qfDump('msgCopyListener::OnProgress(' + progress + ', ' + progressMax + ')');
		},

		OnStartCopy: function ()
		{
			if (this.debug)
				qfDump('msgCopyListener::OnStartCopy()');
		},

		OnStopCopy: function (status)
		{
			if (this.debug)
				qfDump('msgCopyListener::OnStopCopy(' + status + ')');
			
			quickfile.onMoveComplete(status);
		},
		
		SetMessageKey: function (key)
		{
			if (this.debug)
				qfDump('msgCopyListener::SetMessageKey(' + key + ')');
		}
	}
};
